@@ -133,11 +133,17 @@ typedef union
 #define HILDON_APPVIEW_WIDTH           720
 
 
+#define HILDON_RX_51_APPVIEW_HEIGHT          420
+#define HILDON_RX_51_APPVIEW_WIDTH           800
+
+
 //todo: make this a class instance, we might end up having multiple canvases..
 extern liqcanvas canvas;
 
 int           liqcanvas_isopen();
 
+int liqcanvas_settitle(char *newtitle);
+
 //#######################################################
 // allocate using your own custom window
 // i expect this to fail with multiple instances
