@@ -145,6 +145,11 @@ liqcell *	liqcell_getlinkchild(liqcell *self);
 liqcell *	liqcell_getlinkprev_visual(liqcell *self);
 liqcell *	liqcell_getlinknext_visual(liqcell *self);
 liqcell *	liqcell_getlinkchild_visual(liqcell *self);
+
+liqcell *	liqcell_getlinkprev_visible(liqcell *self);
+liqcell *	liqcell_getlinknext_visible(liqcell *self);
+liqcell *	liqcell_getlinkchild_visible(liqcell *self);
+
 //######################################################################### searching
 
 //liqcell*  liqcell_findfirst(liqcell *self,char *query);
