@@ -41,12 +41,18 @@ int liqcell_showdebugboxes=0;
 
 
 
+
 liqimage *easypaint_isloading_image = NULL;
 liqimage *easypaint_barcode_image = NULL;
+liqimage *easypaint_backgrain_image = NULL;
 
 //qr_barcode.png
 
 
+extern int liqcell_easyrun_cursor_on_screen;
+extern int liqcell_easyrun_cursorflashcount;
+extern int liqcell_easyrun_fingerpressed;
+
 
 
 
@@ -82,6 +88,7 @@ int newprio = threadpriority;//20;
 
 
 
+
 static int mainthread_inprogress=0;
 
 
@@ -92,14 +99,15 @@ void *mainthread(void* mainthread_data)
 
 	//liqapp_sleep(100 + (rand() % 4000));
 	//liqapp_sleep(100 + (rand() % 2000));
-	
-	
+	liqcell *self = (liqcell *)mainthread_data;
+	liqcell *par=liqcell_getlinkparent(self);
 do
 {
-	//liqapp_sleep(10 + (rand() % 100));
-	liqapp_sleep(10 + (50));
+//	liqapp_sleep(10 + (rand() % 100));
+	//liqapp_sleep(10 + (50));
+    liqapp_sleep(100);
 }
-while(mainthread_inprogress>1);
+while( (mainthread_inprogress>1) || (par && (par->kineticx || par->kineticy)) );
 
 	mainthread_inprogress++;
 	
@@ -107,14 +115,13 @@ while(mainthread_inprogress>1);
 	
 
 	//
-	liqcell *self = (liqcell *)mainthread_data;
 
 	// we are here to load the image named on self and then finish
 
 
 	liqimage *img = liqcell_getimage(self);
 
-	if(img == easypaint_isloading_image)
+	//if(img == easypaint_isloading_image)
 	{
 
 
@@ -147,8 +154,10 @@ while(mainthread_inprogress>1);
 
 
 				char longbuf[2048];
+                
+                snprintf(longbuf,sizeof(longbuf), "%s/liqbasecache/%s",app.userdatapath, cachefn );
 
-				snprintf(longbuf,sizeof(longbuf), "liqbasecache/%s", cachefn );
+				//snprintf(longbuf,sizeof(longbuf), "liqbasecache/%s", cachefn );
 
 				strncpy(cachefn,longbuf,1024);
 
@@ -195,10 +204,13 @@ while(mainthread_inprogress>1);
 					
 					liqapp_log("http about to download '%s' into '%s'",fn,cachefn);
 					// doesn't exist yet
-					if(!liqapp_pathexists("liqbasecache"))
+                    char xbuf[FILENAME_MAX]={0};
+                    snprintf(xbuf,FILENAME_MAX,"%s/liqbasecache",app.userdatapath);
+					if( !liqapp_pathexists(xbuf) )
 					{
 						// hack to try to make sure theres a liqbasecache folder
-						system("mkdir liqbasecache");
+                        snprintf(xbuf,FILENAME_MAX,"mkdir %s/liqbasecache",app.userdatapath);
+						system(xbuf);
 					}
 
 
@@ -245,6 +257,9 @@ while(mainthread_inprogress>1);
 				{
 					liqcell_setsize(self,imgnew->width,imgnew->height);
 				}
+                
+                
+                liqcell_handlerrun(self,"imageloaded",NULL);    
 
 
 
@@ -283,7 +298,12 @@ int liqcell_threadloadimage(liqcell *self)
 				{
 					liqcell_hold(self);
 					// only if we have a valid replacer can we do this trickery :)
-					liqcell_setimage(self, liqimage_hold(easypaint_isloading_image) );
+                    
+                    if(liqcell_getimage(self)==NULL)
+                    
+                        liqcell_setimage(self, liqimage_hold(easypaint_isloading_image) );
+                        
+                        
 					// now we must start the thread off
 					pthread_t 		tid;
 					
@@ -300,7 +320,7 @@ int liqcell_threadloadimage(liqcell *self)
 						return;
 					}
 				}
-
+	return 0;
 }
 
 
@@ -331,15 +351,24 @@ static float calcaspect(int captionw,int captionh,int availw,int availh)
 }
 
 
-unsigned int decodecolor(char *source,unsigned char *ry,unsigned char *ru,unsigned char *rv)
+unsigned int decodecolor(char *source,unsigned char *ry,unsigned char *ru,unsigned char *rv,unsigned char *ra,unsigned char *rc)
 {
 	char inbuf[1024];
 	snprintf(inbuf,1024,source);
 	char *indat=inbuf;
+    char *s0=NULL;  // used for class
 	char *s1=NULL;
 	char *s2=NULL;
 	char *s3=NULL;
-	//char *s4=NULL;  // used for alpha (not in place..)
+	char *s4=NULL;  // used for alpha
+    
+    if(*indat=='x')
+    {
+        // capture xrgb()  xrgba()  xyuv()  xyuva()
+        s0 = indat;
+        indat++;
+    }
+    
 	if(strncmp(indat,"rgb(",4) == 0 )
 	{
 		indat+=4;
@@ -353,22 +382,21 @@ unsigned int decodecolor(char *source,unsigned char *ry,unsigned char *ru,unsign
 		while(*indat>='0' && *indat<='9')indat++;
 		if(*indat++!=')') return 0;
 		if(s1==s2 || s2==s3) return 0;
-		s2[-1]=0;
-		s3[-1]=0;
-		indat[-1]=0;
+		//s2[-1]=0;		// 20090816_101632 lcuk : leave this nondestructive. atoi will know when it reaches the non chars
+		//s3[-1]=0;
+		//indat[-1]=0;
 		//liqapp_log("%s %i,%i,%i",source,atoi(s1),atoi(s2),atoi(s3) );
 		int R=atoi(s1);
 		int G=atoi(s2);
 		int B=atoi(s3);
-		//ry=atoi(s1);
-		//ru=atoi(s2);
-		//rv=atoi(s3);
 			// convert RGB -> YUV
 			//http://msdn.microsoft.com/en-us/library/ms893078.aspx
 			// yes, microsoft are useful :)
 		*ry = ( (  66 * R + 129 * G +  25 * B + 128) >> 8) +  16;
 		*ru = ( ( 112 * R -  94 * G -  18 * B + 128) >> 8) + 128;
 		*rv = ( ( -38 * R -  74 * G + 112 * B + 128) >> 8) + 128;
+		*ra = 255;
+        *rc = (s0==NULL);
 		return 1;
 	}
 	if(strncmp(indat,"yuv(",4) == 0 )
@@ -384,13 +412,78 @@ unsigned int decodecolor(char *source,unsigned char *ry,unsigned char *ru,unsign
 		while(*indat>='0' && *indat<='9')indat++;
 		if(*indat++!=')') return 0;
 		if(s1==s2 || s2==s3) return 0;
-		s2[-1]=0;
-		s3[-1]=0;
-		indat[-1]=0;
+		//s2[-1]=0;		// 20090816_101632 lcuk : leave this nondestructive. atoi will know when it reaches the non char
+		//s3[-1]=0;
+		//indat[-1]=0;
 		//liqapp_log("%s %i,%i,%i",source,atoi(s1),atoi(s2),atoi(s3) );
 		*ry=atoi(s1);
 		*ru=atoi(s2);
 		*rv=atoi(s3);
+		*ra=255;
+        *rc = (s0==NULL);
+		return 1;
+	}
+	//### alpha variations
+	if(strncmp(indat,"rgba(",5) == 0 )
+	{
+		indat+=5;
+		s1=indat;
+		while(*indat>='0' && *indat<='9')indat++;
+		if(*indat++!=',') return 0;
+		s2=indat;
+		while(*indat>='0' && *indat<='9')indat++;
+		if(*indat++!=',') return 0;
+		s3=indat;
+		while(*indat>='0' && *indat<='9')indat++;
+		if(*indat++!=',') return 0;
+		s4=indat;
+		while(*indat>='0' && *indat<='9')indat++;
+		if(*indat++!=')') return 0;
+		if(s1==s2 || s2==s3 || s3==s4) return 0;
+		//s2[-1]=0;		// 20090816_101632 lcuk : leave this nondestructive. atoi will know when it reaches the non chars
+		//s3[-1]=0;
+		//s4[-1]=0;
+		//indat[-1]=0;
+		//liqapp_log("rgba '%s' %i,%i,%i,%i",source,atoi(s1),atoi(s2),atoi(s3), atoi(s4));
+		int R=atoi(s1);
+		int G=atoi(s2);
+		int B=atoi(s3);
+			// convert RGB -> YUV
+			//http://msdn.microsoft.com/en-us/library/ms893078.aspx
+			// yes, microsoft are useful :)
+		*ry = ( (  66 * R + 129 * G +  25 * B + 128) >> 8) +  16;
+		*ru = ( ( 112 * R -  94 * G -  18 * B + 128) >> 8) + 128;
+		*rv = ( ( -38 * R -  74 * G + 112 * B + 128) >> 8) + 128;
+		*ra = atoi(s4);
+        *rc = (s0==NULL);
+		return 1;
+	}
+	if(strncmp(indat,"yuva(",5) == 0 )
+	{
+		indat+=5;
+		s1=indat;
+		while(*indat>='0' && *indat<='9')indat++;
+		if(*indat++!=',') return 0;
+		s2=indat;
+		while(*indat>='0' && *indat<='9')indat++;
+		if(*indat++!=',') return 0;
+		s3=indat;
+		while(*indat>='0' && *indat<='9')indat++;
+		if(*indat++!=',') return 0;
+		s4=indat;
+		while(*indat>='0' && *indat<='9')indat++;
+		if(*indat++!=')') return 0;
+		if(s1==s2 || s2==s3 || s3==s4) return 0;
+		//s2[-1]=0;		// 20090816_101632 lcuk : leave this nondestructive. atoi will know when it reaches the non chars
+		//s3[-1]=0;
+		//s4[-1]=0;
+		//indat[-1]=0;
+		//liqapp_log("yuva '%s' %i,%i,%i,%i",source,atoi(s1),atoi(s2),atoi(s3), atoi(s4));
+		*ry=atoi(s1);
+		*ru=atoi(s2);
+		*rv=atoi(s3);
+		*ra=atoi(s4);
+        *rc = (s0==NULL);
 		return 1;
 	}
 	return 0;
@@ -627,6 +720,10 @@ __tz_one("paintdone");
 	unsigned char bcy=255;
 	unsigned char bcu=128;
 	unsigned char bcv=128;
+	unsigned char bca=255;	// 20090816_101834 lcuk : new..
+    unsigned char bcc=0;    // Sun Aug 30 23:21:52 2009 lcuk : new class 0==std 1=xrgb(). mmm date format has been lost
+    
+
 
 
 
@@ -636,9 +733,35 @@ __tz_one("paintdone");
 		if(t)
 		{
 			//liqapp_log("textcolor :: '%s'",t);
-			if(decodecolor(t, &bcy, &bcu, &bcv ))
+			if(decodecolor(t, &bcy, &bcu, &bcv, &bca, &bcc ))
 			{
-				liqcliprect_drawboxfillcolor(cr,x,y,w,h,bcy,bcu,bcv);
+                // play...
+                if( (!self->image) && (self->classname) && (bca==255) && (bcc==0) )
+                {
+                    if(!easypaint_backgrain_image)
+                    {
+                         easypaint_backgrain_image=liqimage_newfromfile("/usr/share/liqbase/libliqbase/media/backgrain.png",0,0,0);
+                    }
+                    if(easypaint_backgrain_image)
+                    {
+                        //if(bca==255)
+                            liqcliprect_drawimagecolor(cr,easypaint_backgrain_image,x,y,w,h,0);
+                        //else
+                        //    liqcliprect_drawimageblendcolor(cr,easypaint_backgrain_image,x,y,w,h,bca,0);
+                        liqcliprect_drawboxwashcolor(     cr,x,y,w,h, bcu,bcv);
+                    }
+                }
+                
+                else
+                
+                {
+                    //bca=255;
+                    //if(bca==255)      // such a minute difference wont hurt too much
+                    if(bca>=254)
+                        liqcliprect_drawboxfillcolor(     cr,x,y,w,h,bcy,bcu,bcv);
+                    else
+                        liqcliprect_drawboxfillblendcolor(cr,x,y,w,h,bcy,bcu,bcv,bca);
+                }
 			}
 		}
 	}
@@ -722,6 +845,7 @@ __tz_one("backdone");
 	
 	
 	
+	
 __tz_one("imageprep");
 
 	if(self->image)
@@ -738,73 +862,6 @@ __tz_one("imageprep");
 
 		{
 		//	liqapp_log("imagefloat: start '%s' %i,%i",self->name,w,h);
-/*
-
-			//liqapp_log("imagefloat: start");
-			// when floating, the image is scaled 130%
-			int cx=0;					// cell dimensions
-			int cy=0;
-			int cw=liqcell_getw(self);
-			int ch=liqcell_geth(self);
-			
-			int tx=0;					// total dimensions
-			int ty=0;
-			int tw=cw*1.3;
-			int th=ch*1.3;
-			
-			int frw=tw-cw;
-			int frh=th-ch;
-			
-			float fax=0.03 * (float)(rand() % 100 - 50);	// acceleration is random
-			float fay=0.03 * (float)(rand() % 100 - 50);			
-			float fvx=0;									// velocity is some of the accel
-			float fvy=0;	
-			float fpx=0;									// position is the field
-			float fpy=0;
-			
-			//liqapp_log("imagefloat: reading");
-			int rs=sscanf(fstr,"%f,%f,%f,%f",&fvx,&fvy,&fpx,&fpy);
-			
-			//liqapp_log("imagefloat: read, got %i",rs);
-			
-			#define dt 0.2
-			#define maxv 10
-			
-			if(rs!=4){  fvx=0;  fvy=0;  fpx=0;  fpy=0; }
-			
-			//if(rs==4)
-			{
-				
-				
-				fvx += dt * fax;
-				fvy += dt * fay;
-				
-				if(fvx<-maxv){fvx=-maxv; } else{ if(fvx>maxv){fvx=maxv;}}
-				if(fvy<-maxv){fvy=-maxv; } else{ if(fvy>maxv){fvy=maxv;}}
-				
-				fpx += 0.1 * fvx;
-				fpy += 0.1 * fvy;
-				
-				if(fpx<0){ fpx=0;fvx=0;}else{ if(fpx>frw){fpx=frw;fvx=0;} }
-				if(fpy<0){ fpy=0;fvy=0;}else{ if(fpy>frh){fpy=frh;fvy=0;} }
-				
-				//liqapp_log("imagefloat: writing new");
-				
-				liqcell_propsets_printf(self,"imagefloat","%f,%f,%f,%f",fvx,fvy,fpx,fpy);
-				
-				
-				//liqapp_log("imagefloat: getting dims");
-				
-				int rw=w*1.3;
-				int rh=h*1.3;
-				
-				int xx=x- (fpx*w/cw);
-				int yy=y- (fpy*h/ch);
-				
- 
-			}
-
- */
 			
 			{
 
@@ -952,8 +1009,9 @@ __tz_one("sketchprep");
 		//liqcliprect_drawimagecolor(cr,camimage,x,y,w,h,0);
 		if(liqcell_propgeti(self,"sketchediting",0)==1)
 		{
-			//liqapp_log("edit mode ahoy!");
-			liqcliprect_drawsketch(cr,self->sketch,x,y,w,h,4);
+
+            //liqapp_log("edit mode ahoy!");
+            liqcliprect_drawsketch(cr,self->sketch,x,y,w,h,1 + 4);      // full detailed + noaspect
 		}
 		else
 		{
@@ -1023,12 +1081,179 @@ __tz_one("fontprep");
 				}
 
 
+
 				int captionlen = strlen(caption);
 
 
+
+				//############################################################ get cursor
+
+				int tselstart = liqcell_propgeti(  self,"selstart",-1);
+				int tsellength = liqcell_propgeti(  self,"sellength",0);
+				int tcursorpos = liqcell_propgeti(  self,"cursorpos",-1);
+				
+                
+                
+                if(tcursorpos>=0)liqcell_easyrun_cursor_on_screen++;
+				
+
+
+
+				//liqapp_log("easypainta '%s' ss %i, sl %i, cp %i",self->name,tselstart,tsellength,tcursorpos);
+
+
+
+				//##########################
+				// count all the lines	
+				#define linemax 128
+				int lineheight = liqfont_textheight(self->font);
+				char *linestarts[linemax];
+				int lineoffsets[linemax];
+				int linelengths[linemax];
+				int linecount=0;	
+				
+				
+				if( (tcursorpos<0) || (liqcell_propgeti(self,"wordwrap",0)==0) )
+				{
+					//liqapp_log("single %s:%s",self->name,caption);
+					// text field, but not marked multiline
+					// just let it go through as a single item
+					linestarts[linecount]=caption;
+					lineoffsets[linecount]=0;
+					linelengths[linecount]=captionlen;
+					linecount++;
+				}
+				else
+				{
+					//liqapp_log("multi %s:%s",self->name,caption);
+					//
+					char *c = caption;					
+					while(c && *c)
+					{
+						int tl = strlen(c);		// total length remaining
+						
+	
+						int lc = liqfont_textfitinside(self->font, c, w-16 );
+						if(lc==0)lc++;
+						//liqapp_log("easypaint '%s' countX '%s' %i,%i",self->name,c,tl,lc);
+						if(lc<tl)
+						{
+							int le = lc;
+							while(le>0)
+							{
+								switch(c[le-1])
+								{
+									case ' ':
+									case ',':
+									case ';':
+									case ':':
+									case '.':
+										le--;
+										break;
+									default:
+										goto fin;
+								}
+							}
+							fin:
+							if(le>0)lc=le;
+							while(c[lc]==' ')lc++;
+						}
+						
+						
+						
+						
+						linestarts[linecount] = c;
+						lineoffsets[linecount] = c-caption;
+						linelengths[linecount] = lc;
+						linecount++;
+						
+						c=&c[lc];
+	
+						
+					}
+				}
+				
+
+				
+				
+
+
+
+
+
 				int xx=x;
 				int yy=y;
 				int ww=w;
+				int hh=h;
+
+				//############################################################ get textcolor
+				unsigned char tcy=255;
+				unsigned char tcu=128;
+				unsigned char tcv=128;
+				unsigned char tca=128;
+                unsigned char tcc=128;
+				{
+					t = liqcell_propgets(self,"textcolor",NULL);
+					if(t)
+					{
+						//liqapp_log("textcolor :: '%s'",t);
+						decodecolor(t, &tcy, &tcu, &tcv, &tca,&tcc );
+					}
+				}
+
+
+
+
+//############# from here
+
+
+
+int linenum;
+for(linenum=0;linenum<linecount;linenum++)
+{
+	// replace caption
+	char caption[1024]={0};
+	strncpy(caption,linestarts[linenum],linelengths[linenum]);
+	int captionlen=strlen(caption);
+	caption[linelengths[linenum]]=0;
+	
+	// and y
+	x = xx;
+	y = yy + lineheight * linenum;
+	w = ww;
+	
+	
+
+	// take off the length of the previous line
+	int selstart =tselstart;
+	int sellength=tsellength;
+	int cursorpos=tcursorpos;
+	{
+		int p;
+		for(p=linenum-1;p>=0;p--)
+		{
+			cursorpos -= linelengths[p];
+			selstart  -= linelengths[p];			
+		}
+	}
+	if(selstart<0)sellength-=(-selstart);
+	if(sellength<0)sellength=0;
+	
+	
+//	liqapp_log("easypaint line '%s' %i: %3i,%3i '%s'",self->name,linenum,x,y,caption);
+
+	
+	
+	
+				//if(cursorpos>=0)
+				{ x+=8; w-=16; y+=4; h-=8; }
+				if(selstart>captionlen)selstart=captionlen;
+				if(selstart+sellength>captionlen)sellength=captionlen-selstart;
+				if(cursorpos>captionlen)cursorpos=(linenum==linecount-1)? captionlen : -1;
+				if(tselstart>=0)if(selstart<0)selstart=0;
+	
+	
+ 
 
 				//############################################################ size it up
 
@@ -1036,35 +1261,6 @@ __tz_one("fontprep");
 				int fh = liqfont_textheight(self->font);
 
 
-				//############################################################ get cursor
-
-				int selstart = liqcell_propgeti(  self,"selstart",-1);
-				int sellength = liqcell_propgeti(  self,"sellength",0);
-				int cursorpos = liqcell_propgeti(  self,"cursorpos",-1);
-				
-				
-				if(cursorpos>=0){ x+=2; w-=2; }
-				
-				
-				if(selstart>captionlen)selstart=captionlen;
-				if(selstart+sellength>captionlen)sellength=captionlen-selstart;
-				if(cursorpos>captionlen)cursorpos=captionlen;
-
-
-
-
-				//############################################################ get textcolor
-				unsigned char tcy=255;
-				unsigned char tcu=128;
-				unsigned char tcv=128;
-				{
-					t = liqcell_propgets(self,"textcolor",NULL);
-					if(t)
-					{
-						//liqapp_log("textcolor :: '%s'",t);
-						decodecolor(t, &tcy, &tcu, &tcv );
-					}
-				}
 
 
 
@@ -1102,8 +1298,9 @@ __tz_one("fontprep");
 							y+=(h-fh)/2;
 						}
 				//############################################################ draw it now
+				//liqapp_log("easypaint2 '%s' ss %i, sl %i, cp %i",self->name,selstart,sellength,cursorpos);
 
-				if(selstart>=0)
+				if(selstart>=0 && (liqcell_easyrun_getactivecontrol()==self))
 				{
 					if(fw>w)
 					{
@@ -1140,7 +1337,7 @@ __tz_one("fontprep");
 					
 					
 					// with selection
-					if(sellength==0)
+					if(sellength==0 || (liqcell_easyrun_getactivecontrol()==NULL) )
 					{
 						// draw normally, but add caret as well
 						liqcliprect_drawtext_color( cr, self->font,  x,y, caption, tcy,tcu,tcv);
@@ -1159,6 +1356,7 @@ __tz_one("fontprep");
 						char *alline = caption;
 
 
+
 						if(selstart>0)
 						{
 
@@ -1172,6 +1370,7 @@ __tz_one("fontprep");
 							int ttt = liqfont_textwidth(self->font,sel);
 							if(sel)free(sel);
 							liqcliprect_drawboxfillcolor(cr,x,y,ttt,fh,tcy,tcu,tcv);
+							//x=liqcliprect_drawtextn_color(  cr, self->font,  x,y, alline, sellength, 255-tcy,tcv-20,tcu+20);
 							x=liqcliprect_drawtextn_color(  cr, self->font,  x,y, alline, sellength, 255-tcy,tcv,tcu);
 							alline+=sellength;
 						}
@@ -1183,18 +1382,30 @@ __tz_one("fontprep");
 						x=tx;
 					}
 
-					if(cursorpos>=0)
+
+
+					//liqapp_log("easypaint3 '%s' ss %i, sl %i, cp %i",self->name,selstart,sellength,cursorpos);
+
+
+					if(cursorpos>=0 && cursorpos<=captionlen)
 					{
-						if(cursorpos==0 && sellength==0)x-=2;
+                        
+	//					if(cursorpos==0 && sellength==0)x-=2;
+                        
 						//x=xx;
 						char *sel=strndup(caption,cursorpos);
 						int ttt = liqfont_textwidth(self->font,sel);
 						if(sel)free(sel);
-						liqcliprect_drawboxwashcolor(cr,x+ttt-1,y,3,fh,     40,20);
+						//liqcliprect_drawboxwashcolor(cr,x+ttt-1,y,3,fh,     40,20);
+						//liqcliprect_drawboxwashcolor(cr,x+ttt-1,y,3,fh,     128,128);
 						
-						//liqcliprect_drawboxfillcolor(cr,x+ttt-1,y,3,fh,40,40,20);
-						liqcliprect_drawboxfillcolor(cr,x+ttt  ,y,1,fh,255,40,20);
-						//liqcliprect_drawboxfillcolor(cr,x+ttt+1,y,1,fh,40,40,20);
+						//liqcliprect_drawboxfillcolor(cr,x+ttt  ,y,1,fh,255,40,20);
+                     //   liqapp_log("cursorx %i:%i",liqcell_easyrun_cursorflashcount,(liqcell_easyrun_cursorflashcount & 1));
+                        if( (liqcell_easyrun_cursorflashcount & 1) == 0 )
+                            liqcliprect_drawboxfillcolor(cr,x+ttt  ,y,1,fh,0,128,128);
+                        else
+                            liqcliprect_drawboxfillcolor(cr,x+ttt  ,y,1,fh,255,128,128);
+                        
 
 					}
 					
@@ -1208,9 +1419,13 @@ __tz_one("fontprep");
 				}
 
 
+
+//############ to here
+}
 				x=xx;
 				y=yy;
 				w=ww;
+				h=hh;
 
 			}
 		}
@@ -1268,7 +1483,7 @@ __tz_one("overlaydone");
 	if(t)
 	{
 		//liqapp_log("textcolor :: '%s'",t);
-		if(decodecolor(t, &bcy, &bcu, &bcv ))
+		if(decodecolor(t, &bcy, &bcu, &bcv , &bca,&bcc))
 		{
 
 			liqcliprect_drawboxlinecolor(cr,x,y,w,h,bcy,bcu,bcv);
@@ -1331,6 +1546,39 @@ __tz_one("scrolldone");
 __tz_one("disablerdone");
 
 
+	//t = liqcell_propgets(self,"bordercolor",NULL);
+	//if(t)
+    if(liqcell_getselected(self))
+	{
+		//liqapp_log("textcolor :: '%s'",t);
+		//if(decodecolor(t, &bcy, &bcu, &bcv , &bca))
+        if(w<400 && h<240)  // shock!
+		{
+
+    		liqcliprect_drawboxwashcolor(     cr,x,y,w,h, 60,240);
+			liqcliprect_drawboxlinecolor(     cr,x,y,w,h,200,20,240);
+		}
+	}
+
+
+__tz_one("selecteddone");
+
+	//t = liqcell_propgets(self,"bordercolor",NULL);
+	//if(t)
+    if(liqcell_easyrun_fingerpressed && (liqcell_easyrun_getactivecontrol()==self))
+	{
+		//liqapp_log("textcolor :: '%s'",t);
+		//if(decodecolor(t, &bcy, &bcu, &bcv , &bca))
+        if(w<400 && h<240)  // shock!
+		{
+
+    		liqcliprect_drawboxwashcolor(     cr,x,y,w,h, 60,80);
+			liqcliprect_drawboxlinecolor(     cr,x,y,w,h,200,20,40);
+		}
+	}
+
+
+__tz_one("presseddone");
 
 
 
@@ -1361,7 +1609,7 @@ __tz_one("disablerdone");
 		if(t)
 		{
 			//liqapp_log("textcolor :: '%s'",t);
-			decodecolor(t, &bcy, &bcu, &bcv );
+			decodecolor(t, &bcy, &bcu, &bcv, &bca,&bcc );
 
 			liqcliprect_drawboxlinecolor(cr,x,y,w,h,bcy,bcu,bcv);
 
