@@ -45,6 +45,9 @@
 #include "liqcell_prop.h"
 #include "liqcell_easyrun.h"
 #include "liqcell_easypaint.h"
+
+#include "liqcell_historystore.h"
+
 #include "vgraph.h"
 
 
@@ -53,9 +56,105 @@
 
 
 
+void liqapp_sleep_real(int millseconds)
+{
+	unsigned long 	ft1=liqapp_GetTicks();
+	unsigned long 	ft2=0;
+	
+			while( ( (ft2=liqapp_GetTicks()) - ft1) < millseconds)
+			{
+				// lets just sleep for a short while (doesnt have to be THAT precise, just stop silly speed overruns)
+				//liqapp_log("sleeping %i",millseconds - (ft2-ft1));
+				liqapp_sleep(millseconds - (ft2-ft1));
+			}
+			
+			//liqapp_log("sleep real %lu,%lu,%lu",ft1,ft2,ft2-ft1);
+			
+			//ft1=ft2;
+}
+
+
+
 extern int liqcell_showdebugboxes;
 
 
+liqcell * liqcell_easyrun_currentdialog=NULL;;
+
+int liqcell_easyrun_activecontrol=NULL;
+int liqcell_easyrun_fingerpressed=0;
+
+
+//########################################################################
+//########################################################################
+//########################################################################
+
+
+
+#define liqcell_easyrunstack_total 128
+
+static struct liqcell_easyrunstack
+{
+	liqcell *runself;		// the actual item we were asked to run
+	vgraph *rendergraph;
+	liqcellpainteventargs *paintargs;
+	liqcellmouseeventargs *mouseargs;
+	liqcellkeyeventargs   *keyargs;
+	liqcellclickeventargs *clickargs;
+	liqcell *hit;
+	int hitx;
+	int hity;
+	int hitw;
+	int hith;
+}
+	       liqcell_easyrunstack[liqcell_easyrunstack_total];
+static int liqcell_easyrunstack_used=-1;
+	
+//########################################################################
+//########################################################################
+//########################################################################
+
+
+	
+	
+
+
+	//######################################################## sillt internal thread function should exist only in the lifetime of here
+	int liqcell_easyrun_cursor_on_screen=0;		// cleared before general render, set to 1 within easypaint if a cursor is required
+	int liqcell_easyrun_cursorflashcount=0;		// incremented every 0.5seconds
+	void *liqcell_easyrun_cursorflashingthread_function(void *context)
+	{
+	
+		//liqapp_sleep(100 + (rand() % 4000));
+		//liqapp_sleep(100 + (rand() % 2000));
+		
+		while(1)
+		{
+			
+			int ison=liqcell_easyrun_cursor_on_screen;
+			liqapp_sleep_real(500);
+			if((liqcell_easyrun_cursor_on_screen > ison) && (liqcell_easyrunstack_used>=0))
+			{
+			//	liqapp_log("cursor t %i,%i: %s",liqcell_easyrun_cursorflashcount,  ison,liqcell_easyrunstack[liqcell_easyrunstack_used].runself->name );
+				liqcell_easyrun_cursorflashcount++;
+				
+				liqcell_setdirty( liqcell_easyrunstack[liqcell_easyrunstack_used].runself ,1);
+				
+				
+
+			}
+		}
+		pthread_exit(0);
+		return NULL;
+	}
+
+
+
+// arghh..
+liqcell * liqcell_easyrun_getactivecontrol()
+{
+	return liqcell_easyrun_activecontrol;
+}
+
 
 static void liqcellmouseeventargs_stroke_start(liqcellmouseeventargs *self,int mx,int my,int mz)
 {
@@ -182,7 +281,7 @@ liqcell *liqcell_easyhittest(liqcell *self,  int mx,int my,int *hitx,int *hity)
 //char buff[256];
 //liqcell_getqualifiedname(self,buff,256);
 	//liqapp_log("in '%s' : starting",buff);
-
+	if(self->enabled==0)return NULL;
 	if(mx < self->x) return NULL;
 	if(my < self->y) return NULL;
 	mx-=self->x;
@@ -326,6 +425,7 @@ static void savethumb(liqcell *cell)
 
 liqcell * toolclick(liqcell *vis)
 {
+	
 	liqcell *self = liqcell_quickcreatewidget("tools","form", 800,480);
 
 	if(self)
@@ -363,22 +463,7 @@ liqcell * toolclick(liqcell *vis)
 		liqcell *b;
 		
 		int hh = 480-64;
-
-		b = liqcell_quickcreatevis("power","liqbattery",  800-64-56,0,   56,56 );
-		//liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		//liqcell_handleradd_withcontext(b,    "click",   tool_help_click, self);
-		//liqcell_propsets(  b,    "backcolor", "rgb(0,100,0)" );
-		liqcell_child_append( self, b );
-
-		b = liqcell_quickcreatevis("clock","container",  800-64-56-56-56,0,   56+56,56 );
-		//liqcell_setcontent(b, liqcell_quickcreatevis("clock","ciroclock_minutes",  0,0,0,0 ) );
-		liqcell_setcontent(b, liqcell_quickcreatevis("clock","ciroclock",  0,0,0,0 ) );
-		//liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		//liqcell_handleradd_withcontext(b,    "click",   tool_help_click, self);
-		//liqcell_propsets(  b,    "backcolor", "rgb(0,100,0)" );
-		liqcell_child_append( self, b );
-
-
+/*
 		b = liqcell_quickcreatevis("help","button",  800-50,64+hh*0.0,   50,+hh*0.2 );
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
 		liqcell_handleradd_withcontext(b,    "click",   tool_help_click, self);
@@ -400,17 +485,13 @@ liqcell * toolclick(liqcell *vis)
 		liqcell_propsets(  b,    "backcolor", "rgb(0,100,100)" );
 		liqcell_child_append( self, b );		
 
+*/
 
 
 
-		b = liqcell_quickcreatevis("pic","button",  800-50,64+hh*0.6,   50,hh*0.2);
-		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd_withcontext(b,    "click",   tool_pic_click, self);
-		liqcell_propsets(  b,    "backcolor", "rgb(100,100,0)" );
-		liqcell_child_append( self, b );
-
 
 
+/*
 
 		b = liqcell_quickcreatevis("save","button",  800-50,64+hh*0.8,   50,hh*0.2 );
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
@@ -418,6 +499,16 @@ liqcell * toolclick(liqcell *vis)
 		liqcell_propsets(  b,    "backcolor", "rgb(0,0,100)" );
 		liqcell_child_append( self, b );
 
+*/
+
+
+		b = liqcell_quickcreatevis("pic","button",  800-50,64+hh*0.8,   50,hh*0.2);
+		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
+		liqcell_handleradd_withcontext(b,    "click",   tool_pic_click, self);
+		liqcell_propsets(  b,    "backcolor", "rgb(100,100,0)" );
+		liqcell_child_append( self, b );
+
+
 
 
 		liqcell *content = liqcell_getcontent(vis);
@@ -433,10 +524,10 @@ liqcell * toolclick(liqcell *vis)
 		liqcell *c;
 		
 		
-		c = liqcell_quickcreatevis("contentdraw", "liqtop", 0,56,   800-50,480-56 );
+		c = liqcell_quickcreatevis("contentdraw", "liqsketchedit", 0,56,   800-50,480-56 );
 		//liqcell_handleradd_withcontext( c,    "click",   toolitem_click,self);
 		liqcell_child_insert( self, c );
-	
+
 		c = liqcell_quickcreatevis("content", NULL, 0,56,   800-50,480-56 );
 		liqcell_setcontent( c, vis );
 		liqcell_setenabled( c, 0 );		// make sure it has fadeout
@@ -618,14 +709,13 @@ liqcell *liqcell_findfirsthandler(liqcell*root,char *handlername)
 
 
 
-
 //########################################################################
 //########################################################################
 //########################################################################
 
 int liqcell_easyrun_depth=0;
 
-
+int liqcell_easyrun_hide_tools = 0;
 
 /**
  * Main program event handler.
@@ -644,11 +734,33 @@ int liqcell_easyrun(liqcell *self)
 		return -1;
 	}
 	
+	
+	if(liqcell_easyrunstack_used >= liqcell_easyrunstack_total)
+	{
+		liqapp_log("liqcell easyrun cannot continue, max easyrunstack level reached");
+		return -1;
+		
+	}
+
+	
+	if( liqcell_easyrun_depth == 0 )
+	{
+		// first cell, ask if tools should be globally available
+		liqcell_easyrun_hide_tools = liqcell_propgeti(self,"easyrun_hidetools",0);
+	}
+	
+	
 	// set the 
 	//liqcell_propseti(self,"dialog_complete",0);
 	liqcell_propseti(self,"dialog_running",1);
 	
 	liqcell_easyrun_depth++;
+    
+    
+    liqcanvas_settitle( liqcell_getcaption(self) );
+	
+	
+	
 	
 	// what i should do is resize the contents to match the frame
 	// - actually, that may happen more than once per session
@@ -693,8 +805,12 @@ int idle_lazyrun_wanted = liqcell_propgeti(self,"idle_lazyrun_wanted",0);
 
 	void easyrun_realtime_reshape()
 	{
+		if(canvas.fullscreen)
+			vgraph_setscaleaspectlock(graph,  1);
+		else
+			vgraph_setscaleaspectlock(graph,  0);
 		//vgraph_setscaleaspectlock(graph,  0);
-		vgraph_setscaleaspectlock(graph,  1);
+		
 		vgraph_setcliprect(graph, NULL );
 		vgraph_settarget(graph,   NULL  );
 		
@@ -712,6 +828,45 @@ int idle_lazyrun_wanted = liqcell_propgeti(self,"idle_lazyrun_wanted",0);
 
 
 
+
+
+
+	liqcell_easyrunstack_used++;
+	// unmissable block of text :)
+	liqcell_easyrunstack[liqcell_easyrunstack_used].runself = self;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].rendergraph = graph;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].hit = NULL;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].hitx = 0;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].hity = 0;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].hitw = 0;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].hith = 0;
+	
+	liqcell_easyrunstack[liqcell_easyrunstack_used].paintargs = NULL;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].mouseargs = NULL;;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].keyargs = NULL;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].clickargs = NULL;
+	
+	
+
+
+
+	
+
+
+	pthread_t 		cursorflashingthread=0;
+	
+	if( (liqcell_easyrunstack_used==1) && (liqapp_pref_checkexists("noflashingcursor")==0) )
+	{
+		int tres=pthread_create(&cursorflashingthread,NULL,liqcell_easyrun_cursorflashingthread_function,self);
+	}
+
+
+	
+
+
+
+
+
 int 			running=1;
 int 			result=0;
 
@@ -750,6 +905,21 @@ liqcellmouseeventargs mouseargs;
 liqcellkeyeventargs keyargs;
 liqcellclickeventargs clickargs;
 	clickargs.newdialogtoopen = NULL;
+	
+	
+	
+	
+	
+	
+	
+	
+	liqcell_easyrunstack[liqcell_easyrunstack_used].paintargs = &paintargs;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].mouseargs = &mouseargs;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].keyargs = &keyargs;
+	liqcell_easyrunstack[liqcell_easyrunstack_used].clickargs = &clickargs;
+	
+	
+
 
 int wx=0;
 int wy=0;
@@ -929,8 +1099,15 @@ waitevent:
 				//result=NULL;
 				break;
 			}
-
-
+			else
+			if( (ev.type == LIQEVENT_TYPE_KEY) && (ev.state==LIQEVENT_STATE_PRESS) && (ev.key.keycode==65421) && ev.key.keymodifierstate==4 )	//CTRL+ENTER
+			{
+				liqapp_log("CTRL+Enter pressed, saving screenshot");
+				
+				savethumb( self );
+				//result=NULL;
+				break;
+			}
 
 			else if( (ev.type == LIQEVENT_TYPE_KEY) )
 			{
@@ -972,6 +1149,10 @@ waitevent:
 				}
 				if(keyhit)
 				{
+					hot=keyhit;	// Thu Aug 20 01:35:30 2009 lcuk : mm
+					liqcell_easyrun_activecontrol = hot;
+					
+					
 					// depending upon the key we should handle it...
 					liqapp_log("hello in keyhit:: : %s",keyhit->name);
 
@@ -1028,6 +1209,9 @@ waitevent:
 			else if(ev.type == LIQEVENT_TYPE_MOUSE && (zoom_in_progress==0)   )// && ev.mouse.pressure==0)
 			{
 				// mouse moving! w00t
+				
+				// lets just make sure we let this variable know
+				liqcell_easyrun_fingerpressed = (ev.mouse.pressure>0);
 
 				// get hold of actual coordinates...
 				int mx=ev.mouse.x;
@@ -1083,6 +1267,8 @@ waitevent:
 						liqcellmouseeventargs_stroke_start(&mouseargs,wx,wy,ev.mouse.pressure);
 
 						mouseargs.hit = hot;
+						
+						liqcell_easyrun_activecontrol = hot;
 
 
 
@@ -1121,7 +1307,8 @@ waitevent:
 				}
 
 
-							if( ((targetsurface->width-omsx)<64) && (omsy<64) && ((targetsurface->width-omex)<64) && (omey<64) )
+							//if( ((targetsurface->width-omsx)<64) && (omsy<64) && ((targetsurface->width-omex)<64) && (omey<64) && (liqcell_easyrun_hide_tools==0) )
+							if( ((canvas.pixelwidth-omsx)<64) && (omsy<64) && ((canvas.pixelwidth-omex)<64) && (omey<64) && (liqcell_easyrun_hide_tools==0) )
 							{
 								if(ev.mouse.pressure!=0)
 								{
@@ -1161,9 +1348,10 @@ waitevent:
 								}
 								
 							}
+						
 
-
-							if( (omsx<64) && ((targetsurface->height-omsy)<64) && (omex<64) && ((targetsurface->height-omey)<64) )
+							//if( (omsx<64) && ((targetsurface->height-omsy)<64) && (omex<64) && ((targetsurface->height-omey)<64) )
+							if( (omsx<64) && ((canvas.pixelheight-omsy)<64) && (omex<64) && ((canvas.pixelheight-omey)<64) )
 							{
 								if(ev.mouse.pressure!=0)
 								{
@@ -1171,6 +1359,7 @@ waitevent:
 								}
 								else
 								{
+                                    hot=NULL;
 									running=0;
 									goto quickfin;
 								}
@@ -1264,6 +1453,9 @@ waitevent:
 									liqapp_log("click run '%s'",vhit->name);
 									if( liqcell_handlerrun(vhit,"click",&clickargs) )
 									{
+                                        
+                                        liqcanvas_settitle( liqcell_getcaption(self) );
+                                        
 										// handled it \o/
 										if(clickargs.newdialogtoopen)
 										{
@@ -1409,6 +1601,8 @@ quickfin:
 					// run the zoom_app :)
 
 					liqcell_easyrun( zoom_app );
+                    
+                    liqcanvas_settitle( liqcell_getcaption(self) );
 
 					zoom_app = NULL;
 
@@ -1460,8 +1654,17 @@ moar:
 		}
 		if(self->dirty && (refreshinprogress==0))// && (dirty==0))
 		{
-			dirty=1;
-			self->dirty=0;
+			
+			if(liqcanvas_eventcount())
+			{
+				// this is a test, "dont draw, events are more important"
+				
+			}
+			else
+			{
+				dirty=1;
+				self->dirty=0;
+			}
 		}
 		if(refreshinprogress==0) if(running==0) break;
 		if(paintargs.runfast) dirty=1;
@@ -1497,6 +1700,10 @@ moar:
 			{
 			//	w=ev.mouse.x;
 			}
+			
+			liqcell_easyrun_activecontrol = hot;
+			//liqcell_easyrun_cursor_on_screen = 0;
+			
 			//liqapp_log("render drawing wh(%i,%i)",w,h);
 			vgraph_drawcell(graph,x,y,w,h,self);
 			//liqapp_log("render adding nav items");
@@ -1528,7 +1735,7 @@ moar:
 				
 					liqcliprect_drawimagecolor(targetcr, infoback , 0,targetsurface->height-48,48,48, 1);
 			}
-			if( infotools )
+			if( (infotools) && (liqcell_easyrun_hide_tools==0) )
 			{
 					liqcliprect_drawimagecolor(targetcr, infotools , targetsurface->width-48,0 ,48,48, 1);
 			}
@@ -1543,10 +1750,32 @@ moar:
 				liqapp_sleep(40 - (ft2-ft1));
 			}
 			ft1=ft2;
-#endif					
+#endif
+
+
+
+
+	static const char *cpufreq_filename = "/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_cur_freq";
+	//static const char *cpufreq_filename = "/sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq";
+	int cpufreq_read(int *freq)
+	{
+		FILE *fd;
+		int rs;
+		*freq=0;
+		fd = fopen(cpufreq_filename, "r");
+		if(fd==NULL){ liqapp_log("cpufreq, cannot open for reading"); return -1;}	
+		rs=fscanf((FILE*) fd,"%i",freq);	
+		fclose(fd);	
+		if(rs != 1){ liqapp_log("cpufreq, cannot read information"); return -2;}
+	}
+
+
+
 			//liqapp_log("render adding framecount");		
 // 20090520_014021 lcuk : show frame information
+
 /*
+
 			static liqfont *infofont=NULL;
 			if(!infofont)
 			{
@@ -1557,12 +1786,14 @@ moar:
 				char *cap=liqcell_getcaption(self);
 				if(!cap || !*cap) cap="[nameless]";
 				char buff[255];
+				int cpufreq=0;
+				//cpufreq_read(&cpufreq);
 				snprintf(buff,sizeof(buff),"liqbase '%s' %3i, %3.3f, %3.3f",cap,framecount, liqapp_fps(tz0,tz1,1) ,liqapp_fps(tzs,tz1,framecount) );
 				//liqapp_log(buff);
 				int hh=liqfont_textheight(infofont);
 				liqcliprect_drawtextinside_color(targetcr, infofont,  0,0, targetsurface->width,hh, buff,0, 255,128,128);
 			}
- */		
+ */	
  		
 			//liqapp_log("render refreshing");
 			
@@ -1618,10 +1849,26 @@ moar:
 		}
 	}
 	liqapp_log("liqcell easyrun complete %i",result);
+	
+	
+	if( (liqcell_easyrunstack_used==1) )//&& (liqapp_pref_checkexists("noflashingcursor")==0) )
+	{
+		if(cursorflashingthread)
+		{
+			pthread_cancel(cursorflashingthread);
+		}
+	}
+	
+	
+	
 	vgraph_release(graph);
 	liqstroke_release(mouseargs.stroke);
 	liqcell_handlerrun(self,"dialog_close",NULL);
 	liqcell_propremovei(self,"dialog_running");
+	liqcell_easyrunstack_used--;
 	liqcell_easyrun_depth--;
+	
+	liqcell_historystore_historythumb(self);			// lol!
+	
 	return result;
 }
